@@ -8,11 +8,7 @@ enum {
 };
 
 struct xt_DNETMAP_tginfo {
-#ifdef __KERNEL__
 	struct nf_nat_ipv4_multi_range_compat prefix;
-#else
-	struct nf_nat_multi_range_compat prefix;
-#endif
 	__u8 flags;
 	__s16 ttl;
 };
