@@ -3,12 +3,13 @@
  *	written by Jan Engelhardt <jengelh [at] medozas de>, 2008 - 2009
  *	placed in the Public Domain
  */
+#include <linux/module.h>
 #include <linux/netfilter.h>
 #include <linux/skbuff.h>
 #include "compat_xtables.h"
 
 static unsigned int
-steal_tg(struct sk_buff **pskb, const struct xt_target_param *par)
+steal_tg(struct sk_buff **pskb, const struct xt_action_param *par)
 {
 	kfree_skb(*pskb);
 	return NF_STOLEN;
